package com.danitheskunk.skunkworks.backends.gl;

import com.danitheskunk.skunkworks.Vec2f;
import com.danitheskunk.skunkworks.Vec2i;

import static org.lwjgl.opengl.GL11.*;
import static org.lwjgl.opengl.GL11.glEnd;
import static org.lwjgl.opengl.GL20.glVertexAttrib2f;

public abstract class Filter {
	private final boolean resizing;
	protected Framebuffer input;
	protected Framebuffer output;
	protected Program program;
	protected Vec2f sizeRatio;

	public Filter(Framebuffer input, Program program, boolean resizing) {
		this.resizing = resizing;
		this.input = input;
		this.program = program;
		output = new Framebuffer(input.getSize());
		calcSizeRatio();
	}

	public void apply() {
		//shouldn't be calculating size ratio every frame,
		//but getting all the callbacks right is hard
		calcSizeRatio();

		output.bind();
		input.bindTexture();
		program.use();
		preRender();
		glViewport(0, 0, output.getSize().getX(), output.getSize().getY());
		glBegin(GL_TRIANGLES);
		//counterclockwise triangles
		glVertex2i(-1,  1);
		glVertex2i( 1, -1);
		glVertex2i(-1, -1);

		glVertex2i( 1, -1);
		glVertex2i(-1,  1);
		glVertex2i( 1,  1);
		glEnd();
	}

	public Framebuffer getOutput() {
		return output;
	}

	protected void preRender() {

	}

	public void setInput(Framebuffer input) {
		this.input = input;
		if(!resizing) {
			output.setSize(input.getSize());
		}
	}

	public void setOutputSize(Vec2i size) {
		if(!resizing) {
			throw new RuntimeException("this filter doesn't allow output resizing");
		}
		output.setSize(size);
	}

	private void calcSizeRatio() {
		sizeRatio = Vec2f.div(output.getSize(), input.getSize());
	}

	public Vec2i translateScreenCoordinate(Vec2i coord) {
		return coord;
	}
}
