package com.danitheskunk.skunkworks.backends.gl;

import com.danitheskunk.skunkworks.IMouse;
import com.danitheskunk.skunkworks.Vec2i;

import java.util.Arrays;

import static org.lwjgl.glfw.GLFW.*;

public class Mouse implements IMouse {
	private final boolean[] stateMouseClicked;
	private final boolean[] stateMouseDown;
	private final long window;
	private Filter scaler;

	public Mouse(long window, Filter scaler) {
		this.window = window;
		stateMouseClicked = new boolean[8];
		stateMouseDown = new boolean[8];
		glfwSetMouseButtonCallback(window, this::mouseButtonCallback);
		this.scaler = scaler;
	}

	public Vec2i getPos() {
		double[] x = {0.0};
		double[] y = {0.0};
		glfwGetCursorPos(window, x, y);
		var mousePos = new Vec2i((int) x[0], (int) y[0]);
		var scaledMousePos = scaler.translateScreenCoordinate(mousePos);
		return Vec2i.max(
			Vec2i.ZERO,
			Vec2i.min(
				Vec2i.sub(scaler.input.getSize(), new Vec2i(1, 1)),
				scaledMousePos
			)
		);
	}

	private void mouseButtonCallback(
		@SuppressWarnings("unused") long window, int button, int action, @SuppressWarnings(
		"unused") int mods
	) {
		switch(action) {
			case GLFW_PRESS -> {
				stateMouseClicked[button] = true;
				stateMouseDown[button] = true;
			}
			case GLFW_RELEASE -> stateMouseDown[button] = false;
		}
	}

	public void setScaler(Filter scaler) {
		this.scaler = scaler;
	}

	public void tick() {
		Arrays.fill(stateMouseClicked, false);
	}

	public boolean isClicked(int button) {
		return stateMouseClicked[button];
	}

	public boolean isDown(int button) {
		return stateMouseDown[button];
	}

}
