package com.danitheskunk.skunkworks.backends.gl;

import com.danitheskunk.skunkworks.Vec2f;
import com.danitheskunk.skunkworks.Vec2i;

import static org.lwjgl.opengl.GL46.glUniform1f;
import static org.lwjgl.opengl.GL46.glUniform2f;

public class ScalerAspect extends Filter {
	private static final String fragmentSource = """
		#version 450
		layout(location = 1) in vec2 texCoord;
		layout(binding = 0) uniform sampler2D tex;
		layout(location = 2) uniform float scale;
		layout(location = 3) uniform vec2 offset;
		layout(location = 4) uniform vec2 sizeRatio;
		out vec4 color;
		void main() {
			vec2 tc = (texCoord-offset)*sizeRatio/scale;
			vec4 col = vec4(0,0,0,0);
			if(tc.x <= 1.0 && tc.y <= 1.0 && tc.x >= 0.0 && tc.y >= 0.0) {
				col = texture(tex, tc);
			}
			if(col.a > 0.0) {
				color = col;
			}
		}
		""";
	private static final String vertexSource = """
		#version 450
		layout(location = 0) in vec2 pos;
		layout(location = 1) out vec2 out_texCoord;
		void main() {
			gl_Position = vec4(pos, 0.0f, 1.0f);
			out_texCoord = vec2(pos.x * 0.5 + 0.5, pos.y * 0.5 + 0.5);
		}
		""";

	public ScalerAspect(Framebuffer input) {
		super(input, new Program(vertexSource, fragmentSource), true);
	}

	@Override
	protected void preRender() {
		float scalex = (float) output.getSize().getX() /
			(float) input.getSize().getX();
		float scaley = (float) output.getSize().getY() /
			(float) input.getSize().getY();
		float scale = Math.min(scalex, scaley);

		float xoff = (
			output.getSize().getX() - input.getSize().getX() * scale
		) / 2 / output.getSize().getX();
		float yoff = (
			output.getSize().getY() - input.getSize().getY() * scale
		) / 2 / output.getSize().getY();
		glUniform1f(program.getUniformLocation("scale"), scale);
		glUniform2f(program.getUniformLocation("offset"), xoff, yoff);
		glUniform2f(
			program.getUniformLocation("sizeRatio"),
			(float)sizeRatio.getX(),
			(float)sizeRatio.getY()
		);
	}

	@Override
	public Vec2i translateScreenCoordinate(Vec2i coord) {
		var scale = Vec2f.div(input.getSize(), output.getSize());
		var scalef = Math.max(scale.getX(), scale.getY());

		var off = Vec2f.div(Vec2i.sub(output.getSize(),
			Vec2f.mul(input.getSize().toVec2f(), 1.0 / scalef).toVec2i()
		).toVec2f(), 2).toVec2i();

		return Vec2i.mul(Vec2i.sub(coord, off), 1.0 / scalef);
	}
}
