package com.danitheskunk.skunkworks.nodes;

import com.danitheskunk.skunkworks.Vec2i;
import com.danitheskunk.skunkworks.gfx.Color;
import com.danitheskunk.skunkworks.gfx.IRenderContext;
import com.danitheskunk.skunkworks.gfx.font.IFont;

import java.util.ArrayList;
import java.util.List;

public class NodeText extends Node {
	private Color color;
	private IFont font;
	private List<String> lines;
	private Vec2i maxSize;
	private String text;

	public NodeText(IFont font, String text) {
		this.font = font;
		maxSize = Vec2i.ZERO;
		lines = new ArrayList<>();
		setText(text);
		color = Color.WHITE;
	}

	private void layoutText() {
		lines.clear();
		if(maxSize == Vec2i.ZERO) {
			lines.add(text);
			return;
		}
		var words = text.split(" ");
		String currentLine = words[0];
		for(int i = 1; i < words.length; ++i) {
			String newLine = currentLine + " " + words[i];
			if(font.getStringWidth(newLine) > maxSize.getX()) {
				lines.add(currentLine);
				currentLine = words[i];
			} else {
				currentLine = newLine;
			}
		}
		lines.add(currentLine);
	}

	@Override
	public void render(IRenderContext rc) {
		Vec2i pos = getAbsolutePos().toVec2i();
		for(var line : lines) {
			rc.drawString(pos, line, font, color);
			pos = Vec2i.add(pos, new Vec2i(0, font.getLineHeight()));
		}
	}

	public void setColor(Color color) {
		this.color = color;
	}

	public void setFont(IFont font) {
		this.font = font;
	}

	public void setMaxSize(Vec2i maxSize) {
		this.maxSize = maxSize;
		layoutText();
	}

	public void setText(String text) {
		this.text = text;
		layoutText();
	}
}
